function [P,Q,T]=sGET(bild)
% function[T,phi,msk]=sGET(bild);
%
% Computes the gradient energy tensor for an image <bild>, using Scharr
% filters (modified Sobel filters).
% <T> contains the tensor components in a 3D array T11,T12,T22.
% <phi> contains the local phase for i1D signals.
% <msk> is true for admissible tensors (positive definite) and false
% otherwise.

% Copyright 2004 Michael Felsberg
% Version 1.0

% $Id: sGET.m 26114 2012-06-26 08:48:06Z mfe $

% creating smoothing filter
s=1/16*[3 10 3];

% computing Scharr filter cascade
[gx,gy]=gradient(bild);
gx=imfilter(gx,s');
gy=imfilter(gy,s);
[Hxx Hxy]=gradient(gx);
[Hyx Hyy]=gradient(gy);
Hxx=imfilter(Hxx,s');
Hxy=imfilter(Hxy,s);
Hyx=imfilter(Hyx,s');
Hyy=imfilter(Hyy,s);
L=Hxx+Hyy;
M=1/2*(Hxy+Hyx);
[Lx,Ly]=gradient(L);
Lx=imfilter(Lx,s');
Ly=imfilter(Ly,s);

% compute tensor coefficients
Te=Hxx.*Hxx+Hxy.*Hyx;
Te(:,:,2)=(Hxx+Hyy).*M;
Te(:,:,3)=Hyx.*Hxy+Hyy.*Hyy;
To=-gx.*Lx;
To(:,:,2)=-1/2*(gx.*Ly+gy.*Lx);
To(:,:,3)=-gy.*Ly;
tro=To(:,:,1)+To(:,:,3);
tre=Te(:,:,1)+Te(:,:,3);
% Mask for admissible tensors
%tol=1;
%msk=((tro>-tol)&(To(:,:,1).*To(:,:,3)-To(:,:,2).^2>-tol*tro.^2));
%msk=msk&((tre>-tol)&(Te(:,:,1).*Te(:,:,3)-Te(:,:,2).^2>-tol*tre.^2));
T=To+Te;
% phase:
o=T(:,:,1)-T(:,:,3)+i*2*T(:,:,2);
o=sqrt(o./(abs(o)+eps));
so=sign(real(o).*gx+imag(o).*gy);
o2=Hxx.*real(o)+Hxy.*imag(o)+i*(Hxy.*real(o)+Hyy.*imag(o));
o2=(o2)./(abs(o2)+eps);
st=sign(real(o).*real(o2)+imag(o).*imag(o2));
%phi=angle(-st.*tre-i*so.*tro);
P=-st.*tre;
Q=-so.*tro.*o;